#
# Copyright (C) 2021 Associated Universities, Inc. Washington DC, USA.
#
# This file is part of NRAO Workspaces.
#
# Workspaces is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# Workspaces is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Workspaces.  If not, see <https://www.gnu.org/licenses/>.
"""
palaver definition: an unnecessarily elaborate or complex procedure
(A.K.A. CASA in a nutshell - fight me)
"""
import argparse
import logging
import os
import pathlib
import sys

from casa_envoy.enums import ProductType
from casa_envoy.interfaces import LauncherIF
from casa_envoy.launchers import CalibrationLauncher, ImagingLauncher, SplitLauncher
from pycapo import CapoConfig

"""
Launch CASA processing jobs via Workspaces
"""

logger = logging.getLogger("casa_envoy")
logger.setLevel(logging.INFO)
logger.addHandler(logging.StreamHandler(sys.stdout))


def _get_settings(cwd: pathlib.Path, args: list) -> dict:
    """
    Determine needed settings for the requested type of CASA processing

    :param cwd: path of current directory
    :param args: arguments identifying the processing type and holding the metadata and PPR files
    :return: dictionary of needed settings
    """
    use_casa = CapoConfig().getboolean("edu.nrao.workspaces.ProcessingSettings.useCasa")
    casa_home = CapoConfig().settings("edu.nrao.archive.workflow.config.CasaVersions").homeForReprocessing
    vlass_home = CapoConfig().settings("edu.nrao.workspaces.ProcessingSettings.CasaVersion").vlass

    parent_path = cwd
    root_dir = str(cwd.parent)
    processing_dir = str(cwd.stem)

    metadata = args[0]
    ppr = args[1]

    return {
        "useCasa": use_casa,
        "homeForReprocessing": casa_home,
        "vlassHome": vlass_home,
        "rootDirectory": root_dir,
        "processingDirectory": processing_dir,
        "parent_path": parent_path,
        "metadata": metadata,
        "ppr": ppr,
    }


def _setup_launcher(path: pathlib.Path, args: argparse.Namespace) -> LauncherIF:
    """
    Setup and return the required launcher

    :param path: path of current directory
    :param args: arguments specifying type of processing requested
    :return: Launcher
    """
    parameters = {}

    if args.parallel is not None:
        parameters["requested_parallel"] = args.parallel

    if args.standard_cal is not None or args.vlass_cal is not None:
        if args.standard_cal:
            parameters = {**parameters, **_get_settings(path, args.standard_cal)}
            if args.restore:
                parameters["product_type"] = ProductType.RESTORE.value
            else:
                parameters["product_type"] = ProductType.STD_CAL.value

        if args.vlass_cal:
            parameters = {
                **parameters,
                **_get_settings(path, args.vlass_cal),
                "product_type": ProductType.VLASS_CAL.value,
            }

        return CalibrationLauncher(parameters)
    elif args.standard_img is not None or args.vlass_img is not None:
        if args.standard_img:
            parameters = {**parameters, **_get_settings(path, args.standard_img)}
            if args.integrated:
                parameters["product_type"] = ProductType.INTEGRATED_CAL_IMG.value
            else:
                parameters["product_type"] = ProductType.STD_IMG.value

        if args.vlass_img:
            parameters = {
                **parameters,
                **_get_settings(path, args.vlass_img),
                "product_type": ProductType.VLASS_IMG.value,
            }

        return ImagingLauncher(parameters)
    elif args.split is not None:
        parameters["product_type"] = ProductType.VLASS_SPLIT.value
        parameters = {**parameters, **_get_settings(path, args.split)}
        return SplitLauncher(parameters)


def arg_parser() -> argparse.ArgumentParser:
    parser = argparse.ArgumentParser(
        description="Workspaces CASA processing launcher",
        formatter_class=argparse.RawTextHelpFormatter,
    )
    parser.add_argument(
        "-c",
        "--standard-cal",
        nargs=2,
        action="store",
        required=False,
        help="run the standard calibration CASA pipeline",
    )
    parser.add_argument(
        "-i",
        "--standard-img",
        nargs=2,
        action="store",
        required=False,
        help="run the standard imaging CASA pipeline",
    )
    parser.add_argument(
        "-vc",
        "--vlass-cal",
        nargs=2,
        action="store",
        required=False,
        help="run a VLASS calibration CASA pipeline",
    )
    parser.add_argument(
        "-vi",
        "--vlass-img",
        nargs=2,
        action="store",
        required=False,
        help="run a VLASS imaging CASA pipeline",
    )
    parser.add_argument(
        "-s",
        "--split",
        nargs=2,
        action="store",
        required=False,
        help="run a PIMS Split CASA pipeline",
    )
    parser.add_argument(
        "-p",
        "--parallel",
        action="store",
        required=False,
        help="run CASA with parallelization (mpicasa). Required input: the number of parallel processes desired",
    )
    parser.add_argument(
        "--restore",
        required=False,
        action="store_true",
        help="run the restore measurement set CASA pipeline, use in conjunction with '-c'",
    )
    parser.add_argument(
        "--integrated",
        required=False,
        action="store_true",
        help="run an integrated calibration-imaging pipeline, use in conjunction with '-i'",
    )
    return parser


def main():
    """
    Setup and Run CASA processing

    :return:
    """
    args = arg_parser().parse_args()

    path = pathlib.Path(os.getcwd())

    launcher = _setup_launcher(path, args)

    launcher.launch_casa()

    # make sure we return to the parent directory after processing
    os.chdir(path)
