#
# Copyright (C) 2021 Associated Universities, Inc. Washington DC, USA.
#
# This file is part of NRAO Workspaces.
#
# Workspaces is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# Workspaces is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Workspaces.  If not, see <https://www.gnu.org/licenses/>.
""" Tests for retrieving products from NGAS by various means """
import http

import pytest
import requests_mock
from productfetcher.exceptions import FetchError, RetryableFetchError
from productfetcher.locations import Cluster, Location, NgasFile, NgasServer

# pylint: disable=E0401, E0402, W0621


LOCALHOST_URL = "http://localhost/RETRIEVE"


@pytest.fixture
def ngas_file(tmp_path) -> NgasFile:
    return NgasFile(
        "fake_ngas_id",
        tmp_path.name,
        "ASDMBinary",
        -1,
        "cksum",
        1,
        100,
        NgasServer("localhost", Location.DSOC, Cluster.DSOC),
    )


@pytest.fixture
def ngams_error() -> str:
    return """<?xml version="1.0" encoding="UTF-8"?>
    <!DOCTYPE NgamsStatus SYSTEM "http://nmngas01.aoc.nrao.edu:7777/RETRIEVE?internal=ngamsStatus.dtd">
    <NgamsStatus>
      <Status CompletionTime="2019-06-21T10:40:59.536" Date="2019-06-21T10:40:59.538" HostId="nmngas01"
              LastRequestStatUpdate="2019-06-21T10:40:59.536"
              Message="NGAMS_ER_RETRIEVE_CMD:4018:ERROR: Incorrect parameter given for RETRIEVE command."
              RequestId="705545" RequestTime="2019-06-21T10:40:59.528" State="ONLINE" Status="FAILURE"
              SubState="IDLE" Version="v4.2.2-ALMA/2013-12-101T16:00:00"/>
    </NgamsStatus>"""


def test_direct_copy(ngas_file: NgasFile, ngams_error):
    """
    Can we retrieve a file from NGAS via the direct-copy plugin?

    :param ngas_file: the file to retrieve
    :param ngams_error: the text of the expected response
    :return:
    """
    # test the happy path
    with requests_mock.Mocker() as mock:
        mock.get(LOCALHOST_URL, status_code=http.HTTPStatus.OK, text="Successfully retrieved")
        ngas_file.direct_copy()
        assert mock.called
        assert mock.call_count == 1

    # let's ensure that a 500 error throws an exception
    with requests_mock.Mocker() as mock:
        mock.get(
            LOCALHOST_URL,
            status_code=http.HTTPStatus.INTERNAL_SERVER_ERROR,
            text=ngams_error,
        )
        with pytest.raises(RetryableFetchError):
            ngas_file.direct_copy()

    # let's ensure that a 404 error throws a non-retryable exception
    with requests_mock.Mocker() as mock:
        mock.get(LOCALHOST_URL, status_code=http.HTTPStatus.NOT_FOUND, text=ngams_error)
        with pytest.raises(FetchError):
            ngas_file.direct_copy()


def test_streaming(ngas_file: NgasFile, ngams_error):
    """
    Can we stream a file from NGAS?

    :param ngas_file: the file to retrieve
    :param ngams_error: the text of the expected response
    :return:
    """
    with requests_mock.Mocker() as mock:
        mock.get(LOCALHOST_URL, status_code=http.HTTPStatus.OK, text="Successfully retrieved")
        ngas_file.stream()
        assert mock.called
        assert mock.call_count == 1

    # let's ensure that a 500 error throws an exception
    with requests_mock.Mocker() as mock:
        mock.get(
            LOCALHOST_URL,
            status_code=http.HTTPStatus.INTERNAL_SERVER_ERROR,
            text=ngams_error,
        )
        with pytest.raises(RetryableFetchError):
            ngas_file.stream()

    # let's ensure that a 404 error throws a non-retryable exception
    with requests_mock.Mocker() as mock:
        mock.get(LOCALHOST_URL, status_code=http.HTTPStatus.NOT_FOUND, text=ngams_error)
        with pytest.raises(FetchError):
            ngas_file.stream()
